# Copyright (c) 2024-present, FriendliAI Inc. All rights reserved.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: response_format.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from __future__ import annotations

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x15response_format.proto\x12\x04orca"\x87\x01\n\x0eResponseFormat\x12\'\n\x04type\x18\x01 \x01(\x0e\x32\x19.orca.ResponseFormat.Type\x12\x13\n\x06schema\x18\x02 \x01(\tH\x00\x88\x01\x01",\n\x04Type\x12\x08\n\x04text\x10\x00\x12\x0f\n\x0bjson_object\x10\x01\x12\t\n\x05regex\x10\x02\x42\t\n\x07_schemab\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "response_format_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_RESPONSEFORMAT"]._serialized_start = 32
    _globals["_RESPONSEFORMAT"]._serialized_end = 167
    _globals["_RESPONSEFORMAT_TYPE"]._serialized_start = 112
    _globals["_RESPONSEFORMAT_TYPE"]._serialized_end = 156
# @@protoc_insertion_point(module_scope)
