# Copyright (c) 2024-present, FriendliAI Inc. All rights reserved.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chat_completions.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from __future__ import annotations

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

from friendli.schema.api.v1.codegen import response_format_pb2 as response__format__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x16\x63hat_completions.proto\x12\x04orca\x1a\x1cgoogle/protobuf/struct.proto\x1a\x15response_format.proto"|\n\x08ToolCall\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12)\n\x08\x66unction\x18\x03 \x01(\x0b\x32\x17.orca.ToolCall.Function\x1a+\n\x08\x46unction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x01(\t"\xa5\x01\n\x07Message\x12\x14\n\x07\x63ontent\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04role\x18\x02 \x01(\t\x12\x11\n\x04name\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x19\n\x0ctool_call_id\x18\x04 \x01(\tH\x02\x88\x01\x01\x12"\n\ntool_calls\x18\x05 \x03(\x0b\x32\x0e.orca.ToolCallB\n\n\x08_contentB\x07\n\x05_nameB\x0f\n\r_tool_call_id"\xac\x01\n\x04Tool\x12\x0c\n\x04type\x18\x01 \x01(\t\x12%\n\x08\x66unction\x18\x02 \x01(\x0b\x32\x13.orca.Tool.Function\x1ao\n\x08\x46unction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x18\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x00\x88\x01\x01\x12+\n\nparameters\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructB\x0e\n\x0c_description"\xf6\x07\n\x18V1ChatCompletionsRequest\x12\x1f\n\x08messages\x18\x01 \x03(\x0b\x32\r.orca.Message\x12\x12\n\x05model\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1e\n\x11\x66requency_penalty\x18\x03 \x01(\x02H\x01\x88\x01\x01\x12\x41\n\nlogit_bias\x18\x04 \x03(\x0b\x32-.orca.V1ChatCompletionsRequest.LogitBiasEntry\x12\x17\n\nmin_tokens\x18\x05 \x01(\x05H\x02\x88\x01\x01\x12\x17\n\nmax_tokens\x18\x06 \x01(\x05H\x03\x88\x01\x01\x12\x0e\n\x01n\x18\x07 \x01(\x05H\x04\x88\x01\x01\x12\x1d\n\x10presence_penalty\x18\x08 \x01(\x02H\x05\x88\x01\x01\x12\x0c\n\x04stop\x18\t \x03(\t\x12\x13\n\x06stream\x18\n \x01(\x08H\x06\x88\x01\x01\x12\x18\n\x0btemperature\x18\x0b \x01(\x02H\x07\x88\x01\x01\x12\x12\n\x05top_p\x18\x0c \x01(\x02H\x08\x88\x01\x01\x12!\n\x14timeout_microseconds\x18\r \x01(\x05H\t\x88\x01\x01\x12\x15\n\x08logprobs\x18\x0e \x01(\x08H\n\x88\x01\x01\x12\x19\n\x0ctop_logprobs\x18\x0f \x01(\x05H\x0b\x88\x01\x01\x12\x12\n\x05top_k\x18\x13 \x01(\x05H\x0c\x88\x01\x01\x12\x1f\n\x12repetition_penalty\x18\x14 \x01(\x02H\r\x88\x01\x01\x12\x0c\n\x04seed\x18\x15 \x03(\x04\x12\x11\n\teos_token\x18\x16 \x03(\x05\x12\x19\n\x05tools\x18\x17 \x03(\x0b\x32\n.orca.Tool\x12\x32\n\x0fresponse_format\x18\x18 \x01(\x0b\x32\x14.orca.ResponseFormatH\x0e\x88\x01\x01\x12\x30\n\x0btool_choice\x18\x19 \x01(\x0b\x32\x16.google.protobuf.ValueH\x0f\x88\x01\x01\x12 \n\x13parallel_tool_calls\x18\x1a \x01(\x08H\x10\x88\x01\x01\x1a\x30\n\x0eLogitBiasEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x42\x08\n\x06_modelB\x14\n\x12_frequency_penaltyB\r\n\x0b_min_tokensB\r\n\x0b_max_tokensB\x04\n\x02_nB\x13\n\x11_presence_penaltyB\t\n\x07_streamB\x0e\n\x0c_temperatureB\x08\n\x06_top_pB\x17\n\x15_timeout_microsecondsB\x0b\n\t_logprobsB\x0f\n\r_top_logprobsB\x08\n\x06_top_kB\x15\n\x13_repetition_penaltyB\x12\n\x10_response_formatB\x0e\n\x0c_tool_choiceB\x16\n\x14_parallel_tool_callsb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chat_completions_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_V1CHATCOMPLETIONSREQUEST_LOGITBIASENTRY"]._loaded_options = None
    _globals["_V1CHATCOMPLETIONSREQUEST_LOGITBIASENTRY"]._serialized_options = b"8\001"
    _globals["_TOOLCALL"]._serialized_start = 85
    _globals["_TOOLCALL"]._serialized_end = 209
    _globals["_TOOLCALL_FUNCTION"]._serialized_start = 166
    _globals["_TOOLCALL_FUNCTION"]._serialized_end = 209
    _globals["_MESSAGE"]._serialized_start = 212
    _globals["_MESSAGE"]._serialized_end = 377
    _globals["_TOOL"]._serialized_start = 380
    _globals["_TOOL"]._serialized_end = 552
    _globals["_TOOL_FUNCTION"]._serialized_start = 441
    _globals["_TOOL_FUNCTION"]._serialized_end = 552
    _globals["_V1CHATCOMPLETIONSREQUEST"]._serialized_start = 555
    _globals["_V1CHATCOMPLETIONSREQUEST"]._serialized_end = 1569
    _globals["_V1CHATCOMPLETIONSREQUEST_LOGITBIASENTRY"]._serialized_start = 1247
    _globals["_V1CHATCOMPLETIONSREQUEST_LOGITBIASENTRY"]._serialized_end = 1295
# @@protoc_insertion_point(module_scope)
